/**
 * \file functions.c
 * \brief The functions for shopeuros_client
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * \version 20090726
 * \author YuGiOhJCJ <yugiohjcj@1s.fr> http://yugiohjcj.1s.fr/
 */

#include "functions.h"

int initMusic(char *path, int volume, Music *music){
	if(music->music!=NULL){
		Mix_FreeMusic(music->music);
		music->music=NULL;
	}
	music->music=Mix_LoadMUS(path);
	if(music->music==NULL){
		fprintf(stderr, "initMusic: error 1: Mix_LoadMUS: '%s'\n", Mix_GetError());
		return(-1);
	}
	Mix_VolumeMusic(volume);
	return(0);
}

int initSample(char *path, int volume, Sample *sample){
	if(sample->sample!=NULL){
		Mix_FreeChunk(sample->sample);
		sample->sample=NULL;
	}
	sample->sample=Mix_LoadWAV(path);
	if(sample->sample==NULL){
		fprintf(stderr, "initSample: error 1: Mix_LoadWAV: '%s'\n", Mix_GetError());
		return(-1);
	}
	Mix_VolumeChunk(sample->sample, volume);
	return(0);
}

int initScore(Window window1, int money, int max_score, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Score *score){
	score->value=money;
	if(sprintf(score->string, "%d / %d ", money, max_score)<0){
		fprintf(stderr, "initScore: error 1: sprintf\n");
		return(-1);
	}
	if(initFont(window1, size, red, green, blue, score->font.path, score->string, font_x, font_y, &score->font)==-1){
		fprintf(stderr, "initScore: error 2: initFont\n");
		return(-1);
	}
	return(0);
}

int initPlayer(Window window1, int money, int max_score, int x, int y, int font_x, int font_y, int font_r, int font_g, int font_b, int font_size, const char *path_player, const char *path_font, Player *player, Uint16 up, Uint16 down, Uint16 left, Uint16 right){
	player->money = money;
	if(initBitmap(x, y, path_player, &player->bitmap)){
		fprintf(stderr, "initPlayer: error 1: initBitmap\n");
		return(-1);
	}
	/* Don't be out of the screen */
	if( (x+player->bitmap.sfc->w) > window1.sfc->w ){
		x=window1.sfc->w-player->bitmap.sfc->w;
	}
	player->bitmap.rct.x = x;
	if( (y+player->bitmap.sfc->h) > window1.sfc->h ){
		y=window1.sfc->h-player->bitmap.sfc->h;
	}
	/* The x and y values must be a factor of 10 */
	x=x/10;
	y=y/10;
	x=x*10;
	y=y*10;

	player->bitmap.rct.x = x;
	player->bitmap.rct.y = y;
	player->bitmap.rct.w = player->bitmap.sfc->w;
	player->bitmap.rct.h = player->bitmap.sfc->h;
	if(initScore(window1, player->money, max_score, font_x, font_y, font_r, font_g, font_b, font_size, &player->score)==-1){
		fprintf(stderr, "initPlayer: error 2: initScore\n");
		return(-1);
	}
	initControls(&player->controls, up, down, left, right);
	return(0);
}

void initControls(Controls *controls, SDLKey up, SDLKey down, SDLKey left, SDLKey right){
	controls->up = up;
	controls->down = down;
	controls->left = left;
	controls->right = right;
}

int initWindow(int width, int height, int bpp, Uint32 flags, const char *title, const char *icon, Window *window){
	window->width = width;
	window->height = height;
	window->bpp = bpp;
	window->flags = flags;
	strcpy(window->title, title);

	window->sfc_icon=SDL_LoadBMP(icon);
	if(window->sfc_icon==NULL){
		fprintf(stderr, "initWindow: error 1: SDL_LoadBMP: '%s'\n", SDL_GetError());
		return(-1);
	}

	window->sfc = SDL_SetVideoMode(width, height, bpp, flags);
	if(window->sfc == NULL){
		fprintf(stderr, "initWindow: error 2: SDL_SetVideoMode: '%s'\n", SDL_GetError());
		return(-1);
	}

	/* SDL_WM_SetCaption must be called after SDL_SetVideoMode,
	   otherwhise it doesn't display the window caption */
	SDL_WM_SetCaption(title, icon);

	/* SDL_WM_SetIcon must be called after SDL_SetVideoMode,
	   otherwhise it does a segmentation error */
	SDL_WM_SetIcon(window->sfc_icon, 0);
	
	/* SDL_ShowCursor must be called after SDL_SetVideoMode,
	   otherwhise it doesn't hide the cursor */
	SDL_ShowCursor(SDL_DISABLE);

	return(0);
}

int initBitmap(int x, int y, const char *path, Bitmap *bitmap){
	strcpy(bitmap->path, path);
	if(bitmap->sfc!=NULL){
		SDL_FreeSurface(bitmap->sfc);
		bitmap->sfc=NULL;
	}
	bitmap->sfc=SDL_LoadBMP(path);
	if(bitmap->sfc==NULL){
		fprintf(stderr, "initBitmap: error 1: SDL_LoadBMP: '%s'\n", SDL_GetError());
		return(-1);
	}
	SDL_SetColorKey(bitmap->sfc, SDL_SRCCOLORKEY, bitmap->sfc->format->Rmask | bitmap->sfc->format->Gmask);
	bitmap->rct.x = x;
	bitmap->rct.y = y;
	bitmap->rct.w = bitmap->sfc->w;
	bitmap->rct.h = bitmap->sfc->h;
	return(0);
}

int initFont(Window window1, int size, Uint8 red, Uint8 green, Uint8 blue, char *path, const char *text, int x, int y, Font *font){
	font->x = x;
	font->y = y;
	font->size = size;
	font->color.r = red;
	font->color.g = green;
	font->color.b = blue;
	font->color.unused = 0;
	strcpy(font->path, path);
	if(font->ttf!=NULL){
		TTF_CloseFont(font->ttf);
		font->ttf=NULL;
	}
	font->ttf = TTF_OpenFont(font->path, font->size);
	if(font->ttf == NULL){
		fprintf(stderr, "initFont: error 1: TTF_OpenFont: '%s'\n", TTF_GetError());
		return(-1);
	}
	font->sfc=TTF_RenderText_Solid(font->ttf, text, font->color);
	if(font->sfc==NULL){
		fprintf(stderr, "initFont: error 2: TTF_RenderText_Solid: '%s'\n", TTF_GetError());
		return(-1);
	}
	/* Horizontal and vertical centering */
	x = x - (font->sfc->w/2);
	y = y - (font->sfc->h/2);
	/* Don't be out of the screen */
	if( (x+font->sfc->w) > window1.sfc->w ){
		x = window1.sfc->w-font->sfc->w;
	}
	if( (y+font->sfc->h) > window1.sfc->h ){
		y = window1.sfc->h-font->sfc->h;
	}
	if( y < 0 ){
		y = 0;
	}
	if( x < 0 ){
		x = 0;
	}

	font->rct.x = x;
	font->rct.y = y;
	return(0);
}

void initVolume(int volume_music, int volume_sample, Volume *volume){
	volume->music = volume_music;
	volume->sample = volume_sample;
}

void initGame(Uint32 time_current, Uint32 time_next_move, Uint32 time_next_object, MenuStatus status_menu, PlayStatus status_play, Uint32 max_time, int max_score, Difficulty difficulty, Mode mode, int move_increment, Game *game){
	game->time_current = time_current;
	game->time_next_move = time_current + time_next_move;
	game->time_next_object = time_current + time_next_object;
	game->time_next_time_update = time_current;
	game->time_next_time_is_up = time_current + max_time;
	game->status_menu=status_menu;
	game->status_play=status_play;
	game->time.value=0;
	game->time.max_value=max_time/1000;
	game->max_time = max_time;
	game->max_score = max_score;	
	game->difficulty = difficulty;
	game->mode = mode;
	game->move_increment = move_increment;
}

void initObject(Bitmap *bitmap, ObjectStatus status, Object *object){
	object->bitmap = bitmap;
	object->status = absent; 
}

Uint32 getVideoModeFlags(int argc, char *argv[]){
	int i;
	for(i=1;i<argc;i++){
		if( (strcmp(argv[i], "-f\0")==0 || strcmp(argv[i], "--fullscreen\0")==0) ){
			return(SDL_SWSURFACE|SDL_FULLSCREEN);
		}
	}
	return(SDL_SWSURFACE);
}

const char* getPath(int argc, char *argv[]){
	int i;
	for(i=1;i<argc;i++){
		if( (strcmp(argv[i], "-p\0")==0 || strcmp(argv[i], "--path\0")==0)
				&& (i+1)<argc){
			return(argv[i+1]);
		}
	}
	return(".\0");
}

int gameStart(Window window1, Player *player1, Player *player2, Player *player3, Object *object1, Bitmap *bmp_nothing, Game *game1, int const_time_next_move, int const_time_next_object){
	initGame(SDL_GetTicks(), const_time_next_move, const_time_next_object, out, played, game1->max_time, game1->max_score, game1->difficulty, game1->mode, game1->move_increment, game1);
	if(initPlayer(window1, 0, game1->max_score, 0, 0, player1->score.font.x, player1->score.font.y, player1->score.font.color.r, player1->score.font.color.g, player1->score.font.color.b, player1->score.font.size, player1->bitmap.path, player1->score.font.path, player1, player1->controls.up, player1->controls.down, player1->controls.left, player1->controls.right)==-1){
		fprintf(stderr, "gameStart: error 1: initPlayer\n");
		return(-1);
	}
	if(initPlayer(window1, 0, game1->max_score, window1.sfc->w, window1.sfc->h, player2->score.font.x, player2->score.font.y, player2->score.font.color.r, player2->score.font.color.g, player2->score.font.color.b, player2->score.font.size, player2->bitmap.path, player2->score.font.path, player2, player2->controls.up, player2->controls.down, player2->controls.left, player2->controls.right)==-1){
		fprintf(stderr, "gameStart: error 2: initPlayer\n");
		return(-1);
	}
	if(initPlayer(window1, 0, game1->max_score, (window1.sfc->w/2), (window1.sfc->h/2), player3->score.font.x, player3->score.font.y, player3->score.font.color.r, player3->score.font.color.g, player3->score.font.color.b, player3->score.font.size, player3->bitmap.path, player3->score.font.path, player3, player3->controls.up, player3->controls.down, player3->controls.left, player3->controls.right)==-1){
		fprintf(stderr, "gameStart: error 3: initPlayer\n");
		return(-1);
	}
	initObject(bmp_nothing, absent, object1);
	return(0);
}

int gameStop(Game *game){
	if(game->status_play==played){
		game->status_menu=in;
		game->status_play=stopped;
		return(0);
	}
	fprintf(stderr, "gameStop: error 1: the game status isn't correct\n");
	return(-1);
}

int updateWindow(SDL_Surface *sfc_window1,
		SDL_Surface *sfc_bitmap_map,
		SDL_Surface *sfc_font_title,
		SDL_Rect rct_font_title,
		SDL_Surface *sfc_font_menu,
		SDL_Rect rct_font_menu,
		SDL_Surface *sfc_font_parent_menu,
		SDL_Rect rct_font_parent_menu,
		SDL_Surface *sfc_player1,
		SDL_Rect rct_player1,
		SDL_Surface *sfc_player2,
		SDL_Rect rct_player2,
		SDL_Surface *sfc_player3,
		SDL_Rect rct_player3,
		SDL_Surface *sfc_score1,
		SDL_Rect rct_score1,
		SDL_Surface *sfc_score2,
		SDL_Rect rct_score2,
		SDL_Surface *sfc_score3,
		SDL_Rect rct_score3,
		SDL_Surface *sfc_time,
		SDL_Rect rct_time,
		Object object1,
		Game game){
			/* Clear the screen */
			SDL_FillRect(sfc_window1, NULL, 0);

			/* Display the map */
			if(SDL_BlitSurface(sfc_bitmap_map, NULL, sfc_window1, NULL)==-1){
				fprintf(stderr, "updateWindow: error 1: SDL_BlitSurface: '%s'\n", SDL_GetError());
				return(-1);
			}

			/* Display the scores */
			if(SDL_BlitSurface(sfc_score1, NULL, sfc_window1, &rct_score1)==-1){
				fprintf(stderr, "updateWindow: error 2: SDL_BlitSurface: '%s'\n", SDL_GetError());
				return(-1);
			}
			if(game.mode==two_players || game.mode==two_players_one_cpu){
				if(SDL_BlitSurface(sfc_score2, NULL, sfc_window1, &rct_score2)==-1){
					fprintf(stderr, "updateWindow: error 3: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
			}
			if(game.mode!=two_players){
				if(SDL_BlitSurface(sfc_score3, NULL, sfc_window1, &rct_score3)==-1){
					fprintf(stderr, "updateWindow: error 4: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
			}

			if(object1.status==present){
				/* Display the object */
				if(SDL_BlitSurface(object1.bitmap->sfc, NULL, sfc_window1, &object1.bitmap->rct)==-1){
					fprintf(stderr, "updateWindow: error 5: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
			}

			/* Display the time */
			if(SDL_BlitSurface(sfc_time, NULL, sfc_window1, &rct_time)==-1){
				fprintf(stderr, "updateWindow: error 6: SDL_BlitSurface: '%s'\n", SDL_GetError());
				return(-1);
			}

			/* Display the players */
			if(rct_player1.y<=rct_player2.y && rct_player2.y<=rct_player3.y){
				if(SDL_BlitSurface(sfc_player1, NULL, sfc_window1, &rct_player1)==-1){
					fprintf(stderr, "updateWindow: error 7: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
				if(game.mode==two_players || game.mode==two_players_one_cpu){
					if(SDL_BlitSurface(sfc_player2, NULL, sfc_window1, &rct_player2)==-1){
						fprintf(stderr, "updateWindow: error 8: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(game.mode!=two_players){
					if(SDL_BlitSurface(sfc_player3, NULL, sfc_window1, &rct_player3)==-1){
						fprintf(stderr, "updateWindow: error 9: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
			}
			if(rct_player1.y<=rct_player3.y && rct_player3.y<=rct_player2.y){
				if(SDL_BlitSurface(sfc_player1, NULL, sfc_window1, &rct_player1)==-1){
					fprintf(stderr, "updateWindow: error 10: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
				if(game.mode!=two_players){
					if(SDL_BlitSurface(sfc_player3, NULL, sfc_window1, &rct_player3)==-1){
						fprintf(stderr, "updateWindow: error 11: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(game.mode==two_players || game.mode==two_players_one_cpu){
					if(SDL_BlitSurface(sfc_player2, NULL, sfc_window1, &rct_player2)==-1){
						fprintf(stderr, "updateWindow: error 12: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
			}
			if(rct_player2.y<=rct_player1.y && rct_player1.y<=rct_player3.y){
				if(game.mode==two_players || game.mode==two_players_one_cpu){
					if(SDL_BlitSurface(sfc_player2, NULL, sfc_window1, &rct_player2)==-1){
						fprintf(stderr, "updateWindow: error 13: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(SDL_BlitSurface(sfc_player1, NULL, sfc_window1, &rct_player1)==-1){
					fprintf(stderr, "updateWindow: error 14: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
				if(game.mode!=two_players){
					if(SDL_BlitSurface(sfc_player3, NULL, sfc_window1, &rct_player3)==-1){
						fprintf(stderr, "updateWindow: error 15: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
			}
			if(rct_player2.y<=rct_player3.y && rct_player3.y<=rct_player1.y){
				if(game.mode==two_players || game.mode==two_players_one_cpu){
					if(SDL_BlitSurface(sfc_player2, NULL, sfc_window1, &rct_player2)==-1){
						fprintf(stderr, "updateWindow: error 16: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(game.mode!=two_players){
					if(SDL_BlitSurface(sfc_player3, NULL, sfc_window1, &rct_player3)==-1){
						fprintf(stderr, "updateWindow: error 17: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(SDL_BlitSurface(sfc_player1, NULL, sfc_window1, &rct_player1)==-1){
					fprintf(stderr, "updateWindow: error 18: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
			}
			if(rct_player3.y<=rct_player1.y && rct_player1.y<=rct_player2.y){
				if(game.mode!=two_players){
					if(SDL_BlitSurface(sfc_player3, NULL, sfc_window1, &rct_player3)==-1){
						fprintf(stderr, "updateWindow: error 19: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(SDL_BlitSurface(sfc_player1, NULL, sfc_window1, &rct_player1)==-1){
					fprintf(stderr, "updateWindow: error 20: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
				if(game.mode==two_players || game.mode==two_players_one_cpu){
					if(SDL_BlitSurface(sfc_player2, NULL, sfc_window1, &rct_player2)==-1){
						fprintf(stderr, "updateWindow: error 21: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
			}
			if(rct_player3.y<=rct_player2.y && rct_player2.y<=rct_player1.y){
				if(game.mode!=two_players){
					if(SDL_BlitSurface(sfc_player3, NULL, sfc_window1, &rct_player3)==-1){
						fprintf(stderr, "updateWindow: error 22: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(game.mode==two_players || game.mode==two_players_one_cpu){
					if(SDL_BlitSurface(sfc_player2, NULL, sfc_window1, &rct_player2)==-1){
						fprintf(stderr, "updateWindow: error 23: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
				}
				if(SDL_BlitSurface(sfc_player1, NULL, sfc_window1, &rct_player1)==-1){
					fprintf(stderr, "updateWindow: error 24: SDL_BlitSurface: '%s'\n", SDL_GetError());
					return(-1);
				}
			}

			switch(game.status_menu){
				case in:
					/* Display the fonts */
					if(SDL_BlitSurface(sfc_font_title, NULL, sfc_window1, &rct_font_title)==-1){
						fprintf(stderr, "updateWindow: error 25: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
					if(SDL_BlitSurface(sfc_font_menu, NULL, sfc_window1, &rct_font_menu)==-1){
						fprintf(stderr, "updateWindow: error 26: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}
					if(SDL_BlitSurface(sfc_font_parent_menu, NULL, sfc_window1, &rct_font_parent_menu)==-1){
						fprintf(stderr, "updateWindow: error 27: SDL_BlitSurface: '%s'\n", SDL_GetError());
						return(-1);
					}

					break;
				case out:
					break;
				default:
					fprintf(stderr, "updateWindow: error 28: the menu status isn't correct\n");
					return(-1);
					break;
			}

			/* Update the display */
			SDL_UpdateRect(sfc_window1, 0, 0, 0, 0);

			return(0);
		}

void freeAll(SDL_Surface *bmp_nothing,
		SDL_Surface *bmp_bill,
		SDL_Surface *bmp_coin,
		SDL_Surface *bmp_map,
		SDL_Surface *player3,
		SDL_Surface *player3_score,
		SDL_Surface *player2,
		SDL_Surface *player2_score,
		SDL_Surface *player1,
		SDL_Surface *player1_score,
		SDL_Surface *window1,
		TTF_Font *text_title,
		TTF_Font *text_menu,
		TTF_Font *text_parent_menu,
		Mix_Chunk *sample_super,
		Mix_Chunk *sample_cool,
		Mix_Chunk *sample_shopeuros,
		Mix_Chunk *sample_menu_switch,
		Mix_Chunk *sample_menu_return,
		Mix_Music *music1,
		Menu *menu1){
	/* Surfaces */
	if(bmp_nothing != NULL){
		SDL_FreeSurface(bmp_nothing);
		bmp_nothing = NULL;
	}
	if(bmp_bill != NULL){
		SDL_FreeSurface(bmp_bill);
		bmp_bill = NULL;
	}
	if(bmp_coin != NULL){
		SDL_FreeSurface(bmp_coin);
		bmp_coin = NULL;
	}
	if(bmp_map != NULL){
		SDL_FreeSurface(bmp_map);
		bmp_map = NULL;
	}
	if(player3 != NULL){
		SDL_FreeSurface(player3);
		player3 = NULL;
	}
	if(player3_score != NULL){
		SDL_FreeSurface(player3_score);
		player3_score = NULL;
	}
	if(player2 != NULL){
		SDL_FreeSurface(player2);
		player2 = NULL;
	}
	if(player2_score != NULL){
		SDL_FreeSurface(player2_score);
		player2_score = NULL;
	}
	if(player1 != NULL){
		SDL_FreeSurface(player1);
		player1 = NULL;
	}
	if(player1_score != NULL){
		SDL_FreeSurface(player1_score);
		player1_score = NULL;
	}
	if(window1 != NULL){
		SDL_FreeSurface(window1);
		window1 = NULL;
	}
	if(text_title != NULL){
		TTF_CloseFont(text_title);
		text_title = NULL;
	}
	if(text_menu != NULL){
		TTF_CloseFont(text_menu);
		text_menu = NULL;
	}
	if(text_parent_menu != NULL){
		TTF_CloseFont(text_parent_menu);
		text_parent_menu = NULL;
	}
	/* Halt samples and musics before freeing them */
	Mix_HaltChannel(-1);
	Mix_HaltMusic();
	/* Chunks */
	if(sample_super!=NULL){
		Mix_FreeChunk(sample_super);
		sample_super=NULL;
	}
	if(sample_cool!=NULL){
		Mix_FreeChunk(sample_cool);
		sample_cool=NULL;
	}
	if(sample_shopeuros!=NULL){
		Mix_FreeChunk(sample_shopeuros);
		sample_shopeuros=NULL;
	}
	if(sample_menu_switch!=NULL){
		Mix_FreeChunk(sample_menu_switch);
		sample_menu_switch=NULL;
	}
	if(sample_menu_return!=NULL){
		Mix_FreeChunk(sample_menu_return);
		sample_menu_return=NULL;
	}
	/* Musics */
	if(music1!=NULL){
		Mix_FreeMusic(music1);
		music1=NULL;
	}
	/* Menus */
	if(menu1!=NULL){
		menuFree(menu1);
		menu1=NULL;
	}
}

void moveUp(SDL_Rect *rct_player, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2){
	if(rct_player->y >= game.move_increment
			&& (rct_player->x!=rct_other_player1.x || (rct_player->y-game.move_increment)!=rct_other_player1.y)
			&& (rct_player->x!=rct_other_player2.x || (rct_player->y-game.move_increment)!=rct_other_player2.y)){
		rct_player->y-=game.move_increment;
	}
}

void moveDown(SDL_Surface *sfc_player, SDL_Rect *rct_player, SDL_Surface *sfc_window, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2){
		if((rct_player->y+sfc_player->h) <= (sfc_window->h-game.move_increment)
				&& (rct_player->x!=rct_other_player1.x || (rct_player->y+game.move_increment)!=rct_other_player1.y)
				&& (rct_player->x!=rct_other_player2.x || (rct_player->y+game.move_increment)!=rct_other_player2.y)){
			rct_player->y+=game.move_increment;
		}
	}

	void moveLeft(SDL_Rect *rct_player, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2){
		if( (rct_player->x >= game.move_increment)
				&& (rct_player->y!=rct_other_player1.y || (rct_player->x-game.move_increment)!=rct_other_player1.x)
				&& (rct_player->y!=rct_other_player2.y || (rct_player->x-game.move_increment)!=rct_other_player2.x)){
			rct_player->x-=game.move_increment;
		}
	}

void moveRight(SDL_Surface *sfc_player, SDL_Rect *rct_player, SDL_Surface *sfc_window, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2){
		if((rct_player->x+sfc_player->w) <= (sfc_window->w-game.move_increment)
				&& (rct_player->y!=rct_other_player1.y || (rct_player->x+game.move_increment)!=rct_other_player1.x)
				&& (rct_player->y!=rct_other_player2.y || (rct_player->x+game.move_increment)!=rct_other_player2.x)){
			rct_player->x+=game.move_increment;
		}
	}

int displayHelpAndVersion(int argc, char *argv[], const char *name, const char *version, const char *description, const char *author){
	int counter;
	for(counter=1;counter<argc;counter++){
		if(strcmp(argv[counter], "-h\0")==0 || strcmp(argv[counter], "--help\0")==0){
			printf("%s %s\n\n"
					"Usage: %s OPTION\n\n"
					"Options:\n"
					" -f, --fullscreen\n\t\tdisplay in fullscreen video mode\n"
					" -h, --help\n\t\tdisplay this help and exit\n"
					" -p, --path PATH\n\t\tuse PATH path to get the data folder\n"
					" -v, --version\n\t\toutput version information and exit\n\n"
					"Examples:\n"
					" %s -h\tdisplay this help and exit\n"
					" %s -v\toutput version information and exit\n\n"
					"Report bugs to %s.\n",
					name,
					description,
					name,
					name,
					name,
					author
			      );
			return(-1);
		}
		if(strcmp(argv[counter], "-v\0")==0 || strcmp(argv[counter], "--version\0")==0){
			printf("%s %s\n\n"
					"Copyright (C) 2009 %s.\n"
					"This is free software; see the source for copying conditions. There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"
					"Written by %s.\n",
					name,
					version,
					author,
					author
			      );
			return(-1);
		}
	}
	return(0);
}

int updateObject(Bitmap *bmp_coin, Bitmap *bmp_bill, Bitmap *bmp_nothing, Object *object, Window window1, Game *game1, const int const_time_next_object){
	int random;
	/* If it is the time to put a new object */
	if(game1->time_current >= game1->time_next_object){
		/* Choose a random object */
		random = 1 + (rand()% 3);
		switch(random){
			case 1:
				/* If it is a coin */
				object->bitmap=bmp_coin;
				object->value=1;
				break;
			case 2:
				/* If it is a bill */
				object->bitmap=bmp_bill;
				object->value=5;
				break;
			case 3:
				/* If it is nothing */
				object->bitmap=bmp_nothing;
				object->value=0;
				break;
			default:
				fprintf(stderr, "updateObject: error 1: the random number isn't correct\n");
				return(-1);
				break;
		}
		/* Choose a random position */
		object->bitmap->rct.x = (rand()% (window1.sfc->w/10)) * game1->move_increment;
		object->bitmap->rct.y = (rand()% (window1.sfc->h/10)) * game1->move_increment;
		/* Update the time for the next object */
		game1->time_next_object = game1->time_current + const_time_next_object;
		/* Update the status of the object */
		object->status=present;
	}
	return(0);
}

int updatePlayer(SDL_Surface *sfc_player3, SDL_Rect *rct_player3, Object object, Game *game, SDL_Rect rct_player1, SDL_Rect rct_player2, SDL_Surface *sfc_window, const int const_time_next_move){
	int random, random_max;
	/* If it is the time to move the player */
	if(game->time_current >= game->time_next_move){
		switch(object.status){
			/* If there is an object */
			case present:
				/* Move to the object */
				switch(game->difficulty){
					case easy:
						random_max = 4;
						break;
					case normal:
						random_max = 3;
						break;
					case hard:
						random_max = 2;
						break;
					default:
						fprintf(stderr, "updatePlayer: error 1: the difficulty status isn't correct\n");
						return(-1);
				}
				random = 1 + (rand()% random_max);
				switch(random){
					case 1:
						/* Move horizontal */
						if(object.bitmap->rct.x > rct_player3->x){
							switch(game->mode){
								case one_player:
									moveRight(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player1);
									break;
								case two_players_one_cpu:
									moveRight(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player2);
									break;
								default:
									/* No CPU player */
									break;
							}
						}
						else{
							switch(game->mode){
								case one_player:
									moveLeft(rct_player3, *game, rct_player1, rct_player1);
									break;
								case two_players_one_cpu:
									moveLeft(rct_player3, *game, rct_player1, rct_player2);
									break;
								default:
									/* No CPU player */
									break;
							}
						}
						break;
					case 2:
						/* Move vertical */
						if(object.bitmap->rct.y > rct_player3->y){
							switch(game->mode){
								case one_player:
									moveDown(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player1);
									break;
								case two_players_one_cpu:
									moveDown(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player2);
									break;
								default:
									/* No CPU player */
									break;
							}
						}
						else{
							switch(game->mode){
								case one_player:
									moveUp(rct_player3, *game, rct_player1, rct_player1);
									break;
								case two_players_one_cpu:
									moveUp(rct_player3, *game, rct_player1, rct_player2);
									break;
								default:
									/* No CPU player */
									break;
							}
						}
						break;
					default:
						/* Don't move */
						break;
				}
				break;
				/* If there is no object */
			case absent:
				/* Move anywhere */
				random = 1 + (rand()% 5);
				switch(random){
					case 1:
						switch(game->mode){
							case one_player:
								moveUp(rct_player3, *game, rct_player1, rct_player1);
								break;
							case two_players_one_cpu:
								moveUp(rct_player3, *game, rct_player1, rct_player2);
								break;
							default:
								/* No CPU player */
								break;
						}
						break;
					case 2:
						switch(game->mode){
							case one_player:
								moveDown(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player1);
								break;
							case two_players_one_cpu:
								moveDown(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player2);
								break;
							default:
								/* No CPU player */
								break;
						}
						break;
					case 3:
						switch(game->mode){
							case one_player:
								moveLeft(rct_player3, *game, rct_player1, rct_player1);
								break;
							case two_players_one_cpu:
								moveLeft(rct_player3, *game, rct_player1, rct_player2);
								break;
							default:
								/* No CPU player */
								break;
						}
						break;
					case 4:
						switch(game->mode){
							case one_player:
								moveRight(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player1);
								break;
							case two_players_one_cpu:
								moveRight(sfc_player3, rct_player3, sfc_window, *game, rct_player1, rct_player2);
								break;
							default:
								/* No CPU player */
								break;
						}
						break;
					case 5:
						/* Don't move */
						break;
					default:
						fprintf(stderr, "updatePlayer: error 2: the random number isn't correct\n");
						return(-1);
						break;
				}
				break;
			default:
				fprintf(stderr, "updatePlayer: error 3: the object status isn't correct\n");
				return(-1);
				break;
		}
		game->time_next_move = game->time_current + const_time_next_move;
	}
	return(0);
}

int updateScore(Window window1, Player *player, Mix_Chunk *sample_cool, Mix_Chunk *sample_super, Object *object, int max_score){
	if( (player->bitmap.rct.x == object->bitmap->rct.x) && (player->bitmap.rct.y == object->bitmap->rct.y) && object->status==present){
		player->score.value+=object->value;
		if(object->value==1){
			/* Play the sample 'cool' on the channel 3 */
			Mix_HaltChannel(3);
			if(Mix_PlayChannel(3, sample_cool, 0)==-1) {
				fprintf(stderr, "updateScore: error 1: Mix_PlayChannel: '%s'\n", Mix_GetError());
				return(-1);
			}
		}
		if(object->value==5){
			/* Play the sample 'super' on the channel 3 */
			Mix_HaltChannel(3);
			if(Mix_PlayChannel(3, sample_super, 0)==-1) {
				fprintf(stderr, "updateScore: error 2: Mix_PlayChannel: '%s'\n", Mix_GetError());
				return(-1);
			}
		}
		object->status=absent;
		if(initPlayer(window1,player->score.value,max_score,player->bitmap.rct.x,player->bitmap.rct.y,player->score.font.x,player->score.font.y,player->score.font.color.r,player->score.font.color.g,player->score.font.color.b,player->score.font.size,player->bitmap.path,player->score.font.path,player,player->controls.up,player->controls.down,player->controls.left,player->controls.right)==-1){
			fprintf(stderr, "updateScore: error 3: initPlayer\n");
			return(-1);
		}
	}
	return(0);
}

int initText(Window window1, const char* string, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Text *text){
	strcpy(text->string, string);
	if(initFont(window1, size, red, green, blue, text->font.path, text->string, font_x, font_y, &text->font)==-1){
		fprintf(stderr, "initText: error 1: initFont\n");
		return(-1);
	}
	return(0);
}

int initTime(Window window1, const int game_time, const int game_time_max, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Time *time){
	sprintf(time->string, "%d / %d s", game_time, game_time_max);
	if(initFont(window1, size, red, green, blue, time->font.path, time->string, font_x, font_y, &time->font)==-1){
		fprintf(stderr, "initTime: error 1: initFont\n");
		return(-1);
	}
	return(0);
}

int updateGameStatus(const int player1_score_value, const int player2_score_value, const int player3_score_value, Game *game1){
	if(game1->max_score<=player1_score_value
			|| game1->max_score<=player2_score_value
			|| game1->max_score<=player3_score_value
			|| game1->time_next_time_is_up<=game1->time_current){
		if(gameStop(game1)==-1){
			fprintf(stderr, "updateGameStatus: error 1: gameStop\n");
			return(-1);
		}
	}
	return(0);
}

int updateTime(Window window1, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Game *game1, const int const_time_next_time_update){
	if(game1->time_current > game1->time_next_time_update){
		game1->time.value += 1;
		if(initTime(window1, game1->time.value, game1->time.max_value, window1.sfc->w, window1.sfc->h, red, green, blue, size, &game1->time)==-1){
			fprintf(stderr, "updateTime: error 1: initTime\n");
			return(-1);
		}
		game1->time_next_time_update += const_time_next_time_update;
	}
	return(0);
}

Menu* initMenu(){
	char string[6+1] = "\0";
	int i = 60000;
	Menu *menu1, *ptr_menu1;
	menu1=menuCreate("Play\0",menuCreate("Options\0",menuCreate("Exit\0",NULL,NULL),menuCreate("Video\0",menuCreate("Audio\0",menuCreate("Control\0",menuCreate("Game\0",NULL,menuCreate("Difficulty\0",menuCreate("Max time",menuCreate("Max score",NULL,NULL),NULL),menuCreate("Easy",menuCreate("Normal",menuCreate("Hard",NULL,NULL),NULL),NULL))),menuCreate("Player 1\0",menuCreate("Player 2",NULL,menuCreate("P2 UP",menuCreate("P2 DOWN",menuCreate("P2 LEFT",menuCreate("P2 RIGHT",NULL,NULL),NULL),NULL),NULL)),menuCreate("P1 UP",menuCreate("P1 DOWN",menuCreate("P1 LEFT",menuCreate("P1 RIGHT",NULL,NULL),NULL),NULL),NULL))),menuCreate("Music\0",menuCreate("Samples\0",NULL,menuCreate("Mute samples\0",menuCreate("Samples volume 1/3\0",menuCreate("Samples volume 2/3\0",menuCreate("Samples volume 3/3\0",NULL,NULL),NULL),NULL),NULL)),menuCreate("Mute music\0",menuCreate("Music volume 1/3\0",menuCreate("Music volume 2/3\0",menuCreate("Music volume 3/3\0",NULL,NULL),NULL),NULL),NULL))),menuCreate("Window\0",menuCreate("Fullscreen\0",NULL,NULL),NULL))),menuCreate("1 player\0",menuCreate("2 players\0",menuCreate("2 players and 1 CPU\0",NULL,NULL),NULL),NULL));
	/* The Max time sub menu */
	ptr_menu1=menuGet(menu1, "Max time");
	sprintf(string,"%d",i);
	ptr_menu1->sub = menuCreate(string,NULL,NULL);
	ptr_menu1=ptr_menu1->sub;
	for(i = 120000;i<=999000;i+=60000){
		sprintf(string,"%d",i);
		ptr_menu1->next = menuCreate(string,NULL,NULL);
		ptr_menu1=ptr_menu1->next;
	}
	/* The Max score sub menu */
	i=10;
	ptr_menu1=menuGet(menu1, "Max score");
	sprintf(string,"%d",i);
	ptr_menu1->sub = menuCreate(string,NULL,NULL);
	ptr_menu1=ptr_menu1->sub;
	for(i = 20;i<=50;i+=10){
		sprintf(string,"%d",i);
		ptr_menu1->next = menuCreate(string,NULL,NULL);
		ptr_menu1=ptr_menu1->next;
	}
	return(menu1);
}
