/**
 * \file functions.h
 * \brief The header for functions.c
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * \version 20090726
 * \author YuGiOhJCJ <yugiohjcj@1s.fr> http://yugiohjcj.1s.fr/
 */

#ifndef functions_h
#define functions_h

#include <SDL/SDL.h> /* For Uint32 and Uint16 */
#include "structures.h" /* For Player */
#include "menu/menu.h" /* For Menu */

/**
 * \fn int initPlayer(Window window1,
 int money,
 int max_score,
 int x,
 int y,
 int font_x,
 int font_y,
 int font_r,
 int font_g,
 int font_b,
 int font_size,
 const char *path_player,
 const char *path_font,
 Player *player,
 Uint16 up,
 Uint16 down,
 Uint16 left,
 Uint16 right)
 * \brief initialize a Player structure
 * \param window1 a Window structure (the window)
 * \param money an integer (the player money)
 * \param max_score an integer (the game maximum score)
 * \param x an interger (the player horizontal position)
 * \param y (the player vertical position)
 * \param font_x an integer (the font horizontal position)
 * \param font_y an integer (the font vertical position)
 * \param font_r an integer (the font red color)
 * \param font_g an integer (the font green color)
 * \param font_b an integer (the font blue color)
 * \param font_size an integer (the font size)
 * \param path_player a string (the path to the player bitmap)
 * \param path_font a string (the font path)
 * \param player a pointer to a Player structure (the player)
 * \param up a SDLKey structure (the key pressed to move up the player)
 * \param down a SDLKey structure (the key pressed to move down the player)
 * \param left a SDLKey structure (the key pressed to move left the player)
 * \param right a SDLKey structure (the key pressed to move right the player)
 * \return an integer (0 if the Player structure is successfully initialized or -1 on error)
 */
int initPlayer(Window window1, int money, int max_score, int x, int y, int font_x, int font_y, int font_r, int font_g, int font_b, int font_size, const char *path_player, const char *path_font, Player *player, Uint16 up, Uint16 down, Uint16 left, Uint16 right);

/**
 * \fn void initControls(Controls *controls,
 SDLKey up,
 SDLKey down,
 SDLKey left,
 SDLKey right)
 * \brief initialize the controls
 * \param controls a pointer to a Controls structure (the controls)
 * \param up a SDLKey structure (the key pressed to move up)
 * \param down a SDLKey structure (the key pressed to move down)
 * \param left a SDLKey structure (the key pressed to move left)
 * \param right a SDLKey structure (the key pressed to move right)
 * \return nothing
 */
void initControls(Controls *controls, SDLKey up, SDLKey down, SDLKey left, SDLKey right);

/**
 * \fn int initScore(Window window1,
 int money,
 int max_score,
 int font_x,
 int font_y,
 Uint8 red,
 Uint8 green,
 Uint8 blue,
 int size,
 Score *score)
 * \brief initialize a Score structure
 * \param window1 a Window structure (the window)
 * \param money an integer (the money)
 * \param max_score an integer (the maximum score)
 * \param font_x an integer (the horizontal player font position)
 * \param font_y an integer (the vertical player font position)
 * \param red an integer (the font red color)
 * \param green an integer (the font red color)
 * \param blue an integer (the font red color)
 * \param size an integer (the font size)
 * \param score a pointer to a Score structure (the score)
 * \return an integer (0 if the Score structure is successfully initialized or -1 on error)
 */
int initScore(Window window1, int money, int max_score, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Score *score);

/**
 * \fn int initWindow(int width,
 int height, 
 int bpp, 
 Uint32 flags, 
 const char *title,
 const char *icon, 
 Window *window)
 * \brief initialize a Window structure
 * \param width an integer (the window width)
 * \param height an integer (the window height)
 * \param bpp an integer (the window bits per pixel)
 * \param flags an integer (the window flags)
 * \param title a string (the window title)
 * \param icon a string (the path to the window icon)
 * \param window a pointer to a SDL SDL_Surface structure (the window surface)
 * \return an integer (0 if the Window structure is successfully initialized or -1 on error)
 */
int initWindow(int width, int height, int bpp, Uint32 flags, const char *title, const char *icon, Window *window);

/**
 * \fn int initBitmap(int x,
 int y,
 const char *path, 
 Bitmap *bitmap)
 * \brief initialize a Bitmap structure
 * \param x an integer (the horizontal bitmap position)
 * \param y an integer (the vertical bitmap position)
 * \param path a string (the path to the bitmap) 
 * \param bitmap a pointer to a Bitmap structure (the bitmap)
 * \return an integer (0 if the Bitmap structure is successfully initialized or -1 on error)
 */
int initBitmap(int x, int y, const char *path, Bitmap *bitmap);

/**
 * \fn int initFont(Window window1,
 int size,
 Uint8 red,
 Uint8 green,
 Uint8 blue,
 char *path,
 const char *text,
 int x,
 int y,
 Font *font)
 * \brief initialize a Font structure
 * \param window1 a Window structure (the window)
 * \param size an integer (the font size)
 * \param red an integer (the red color for the player score font)
 * \param green an integer (the green color for the player score font)
 * \param blue an integer (the blue color for the player score font)
 * \param path a string (the path to the font)
 * \param text a string (the text to display)
 * \param x an integer (the font horizontal position)
 * \param y an integer (the font vertical position)
 * \param font a pointer to a Font structure (the font)
 * \return an integer (0 if the Font structure is successfully initialized or -1 on error)
 */
int initFont(Window window1, int size, Uint8 red, Uint8 green, Uint8 blue, char *path, const char *text, int x, int y, Font *font);

/**
 * \fn void initVolume(int volume_music,
 int volume_sample,
 Volume *volume)
 * \brief initialize a Volume structure
 * \param volume_music an integer (the music volume)
 * \param volume_sample an integer (the sample volume)
 * \param volume a pointer to a Volume structure (the volume) 
 * \return nothing
 */
void initVolume(int volume_music, int volume_sample, Volume *volume);

/**
 * \fn int initSample(char *path,
 int volume,
 Sample *sample)
 * \brief initialize a Sample structure
 * \param path a string (the path to the sample file)
 * \param volume an integer (the sample volume)
 * \param sample a pointer to a Sample structure (the sample)
 * \return an integer (0 if the Sample structure is successfully initialized or -1 on error)
 */
int initSample(char *path, int volume, Sample *sample);

/**
 * \fn int initMusic(char *path,
 int volume,
 Music *music)
 * \brief initialize a Music structure
 * \param path a string (the path to the music file)
 * \param volume volume an integer (the sample volume)
 * \param music a pointer to a Music structure (the music)
 * \return an integer (0 if the Music structure is successfully initialized or -1 on error)
 */
int initMusic(char *path, int volume, Music *music);

/**
 * \fn int initText(Window window1,
 const char* string,
 int font_x,
 int font_y,
 Uint8 red,
 Uint8 green,
 Uint8 blue,
 int size,
 Text *text)
 * \brief initialize a Text structure
 * \param window1 a Window structure (the window)
 * \param string a string (the text value)
 * \param font_x an integer (the font horizontal position)
 * \param font_y an integer (the font vertical position)
 * \param red an integer (the font red color)
 * \param green an integer (the font green color)
 * \param blue an integer (the font blue color)
 * \param size an integer (the font size)
 * \param text a pointer to a Text structure (the text)
 * \return an integer (0 if the Text structure is successfully initialized or -1 on error)
 */
int initText(Window window1, const char* string, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Text *text);

/**
 * \fn int initTime(Window window1,
 const int game_time,
 const int game_time_max,
 int font_x,
 int font_y,
 Uint8 red,
 Uint8 green,
 Uint8 blue,
 int size,
 Time *time)
 * \brief initialize a Time structure
 * \param window1 a Window structure (the window)
 * \param game_time an integer (the game time)
 * \param game_time_max an integer (the maximum game time)
 * \param font_x an integer (the font horizontal position)
 * \param font_y an integer (the font vertical position)
 * \param red an integer (the font red color)
 * \param green an integer (the font green color)
 * \param blue an integer (the font blue color)
 * \param size an integer (the font size)
 * \param time a pointer to a Time structure (the time)
 * \return an integer (0 if the Time structure is successfully initialized or -1 on error)
 */
int initTime(Window window1, const int game_time, const int game_time_max, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Time *time);

/**
 * \fn Menu* initMenu()
 * \brief initialize a Menu structure
 * \return a pointer to a Menu structure (the menu)
 */
Menu* initMenu();

/**
 * \fn void initObject(Bitmap *bitmap,
 ObjectStatus status,
 Object *object)
 * \brief initialize an Object structure
 * \param bitmap a pointer to a Bitmap structure (the bitmap)
 * \param status an ObjectStatus structure (the object status)
 * \param object a pointer to an Object structure (the object)
 * \return nothing
 */
void initObject(Bitmap *bitmap, ObjectStatus status, Object *object);

/**
 * \fn void initGame(Uint32 time_current,
 Uint32 time_next_move,
 Uint32 time_next_object,
 MenuStatus status_menu,
 PlayStatus status_play,
 Uint32 max_time,
 int max_score,
 Difficulty difficulty,
 Mode mode,
 int move_increment,
 Game *game)
 * \brief initialize a Game structure
 * \param time_current an integer (the game current time)
 * \param time_next_move an integer (the game next move time)
 * \param time_next_object an integer (the game next object time)
 * \param status_menu a MenuStatus structure (the game menu status)
 * \param status_play a PlayStatus structure (the game play status)
 * \param max_time an integer (the game maximum time)
 * \param max_score an integer (the game maximum score)
 * \param difficulty a Difficulty structure (the game difficulty)
 * \param mode a Mode structure (the game mode)
 * \param move_increment an integer (the game move increment)
 * \param game a pointer to a Game structure (the game)
 * \return nothing
 */
void initGame(Uint32 time_current, Uint32 time_next_move, Uint32 time_next_object, MenuStatus status_menu, PlayStatus status_play, Uint32 max_time, int max_score, Difficulty difficulty, Mode mode, int move_increment, Game *game);

/**
 * \fn int displayHelpAndVersion(int argc,
 char *argv[],
 const char *name, 
 const char *version, 
 const char *description, 
 const char *author)
 * \brief display the help, the version or nothing (it depends of the arguments)
 * \param argc an integer (the number of arguments)
 * \param argv a pointer to a strings array (the arguments)
 * \param name a string (the name)
 * \param version a string (the version)
 * \param description a string (the description)
 * \param author a string (the author)
 * \return an integer (0 if it doesn't display the help or the version, else -1)
 */
int displayHelpAndVersion(int argc, char *argv[], const char *name, const char *version, const char *description, const char *author);

/**
 * \fn char* getPath(int argc, char *argv[])
 * \brief get the path to the data folder
 * \param argc an integer (the number of arguments)
 * \param argv a pointer to a strings array (the arguments)
 * \return a string (the path)
 */
const char* getPath(int argc, char *argv[]);

/**
 * \fn Uint32 getVideoModeFlags(int argc, char *argv[])
 * \brief get the video mode flags
 * \param argc an integer (the number of arguments)
 * \param argv a pointer to a strings array (the arguments)
 * \return an integer (the flags)
 */
Uint32 getVideoModeFlags(int argc, char *argv[]);

/**
 * \fn int int gameStart(Window window1,
 Player *player1,
 Player *player2,
 Player *player3, 
 Object *object1,
 Bitmap *bmp_nothing,
 Game *game1,
 int const_time_next_move, 
 int const_time_next_object)
 * \brief set game variables in order to start a game
 * \param window1 a Window structure (the window)
 * \param player1 a pointer to a Player structure (the player 1)
 * \param player2 a pointer to a Player structure (the player 2)
 * \param player3 a pointer to a Player structure (the player 3)
 * \param object1 a pointer to a Object structure (the object)
 * \param bmp_nothing a pointer to a Bitmap structure (the bitmap)
 * \param game1 a pointer to a Game structure (the game)
 * \param const_time_next_move an integer (the next move time)
 * \param const_time_next_object an integer (the next object time)
 * \return an integer (0 if the game is successfully started or -1 on error)
 */
int gameStart(Window window1, Player *player1, Player *player2, Player *player3, Object *object1, Bitmap *bmp_nothing, Game *game1, int const_time_next_move, int const_time_next_object);

/**
 * \fn int gameStop(Game *game)
 * \brief set game variables in order to stop a game
 * \param game a pointer to a Game structure (the game)
 * \return an integer (0 if the game is successfully stopped or -1 on error)
 */
int gameStop(Game *game);

/**
 * \fn int updateWindow(SDL_Surface *sfc_window1,
 SDL_Surface *sfc_bitmap_map,
 SDL_Surface *sfc_font_title,
 SDL_Rect rct_font_title,
 SDL_Surface *sfc_font_menu,
 SDL_Rect rct_font_menu,
 SDL_Surface *sfc_font_parent_menu,
 SDL_Rect rct_font_parent_menu,
 SDL_Surface *sfc_player1,
 SDL_Rect rct_player1,
 SDL_Surface *sfc_player2,
 SDL_Rect rct_player2,
 SDL_Surface *sfc_player3,
 SDL_Rect rct_player3,
 SDL_Surface *sfc_score1,
 SDL_Rect rct_score1,
 SDL_Surface *sfc_score2,
 SDL_Rect rct_score2,
 SDL_Surface *sfc_score3,
 SDL_Rect rct_score3,
 SDL_Surface *sfc_time,
 SDL_Rect rct_time,
 Object object1,
 Game game)
 * \brief update the window
 * \param sfc_window1 a pointer to a SDL SDL_Surface structure (the window)
 * \param sfc_bitmap_map a pointer to a SDL SDL_Surface structure (the bitmap)
 * \param sfc_font_title a pointer to a SDL SDL_Surface structure (the title font)
 * \param rct_font_title a SDL SDL_Rect structure (the title font rectangular area)
 * \param sfc_font_menu a pointer to a SDL SDL_Surface structure (the menu font)
 * \param rct_font_menu a SDL SDL_Rect structure (the menu font rectangular area)
 * \param sfc_font_parent_menu a pointer to a SDL SDL_Surface structure (the parent menu font)
 * \param rct_font_parent_menu a SDL SDL_Rect structure (the parent menu font rectangular area)
 * \param sfc_player1 a pointer to a SDL SDL_Surface structure (the player 1)
 * \param rct_player1 a SDL SDL_Rect structure (the player 1 rectangular area)
 * \param sfc_player2 a pointer to a SDL SDL_Surface structure (the player 2)
 * \param rct_player2 a SDL SDL_Rect structure (the player 2 rectangular area)
 * \param sfc_player3 a pointer to a SDL SDL_Surface structure (the player 3)
 * \param rct_player3 a SDL SDL_Rect structure (the player 3 rectangular area)
 * \param sfc_score1 a pointer to a SDL SDL_Surface structure (the score for the player 1)
 * \param rct_score1 a SDL SDL_Rect structure (the score for the player 1 rectangular area)
 * \param sfc_score2 a pointer to a SDL SDL_Surface structure (the score for the player 2)
 * \param rct_score2 a SDL SDL_Rect structure (the score for the player 2 rectangular area)
 * \param sfc_score3 a pointer to a SDL SDL_Surface structure (the score for the player 3)
 * \param rct_score3 a SDL SDL_Rect structure (the score for the player 3 rectangular area)
 * \param sfc_time a pointer to a SDL SDL_Surface structure (the current time surface)
 * \param rct_time a SDL SDL_Rect structure (the current time rectangular area)
 * \param object1 an Object structure (the object)
 * \param game a Game structure (the game)
 * \return an integer (0 if the window is successfully updated or -1 on error)
 */
int updateWindow(SDL_Surface *sfc_window1,
		SDL_Surface *sfc_bitmap_map,
		SDL_Surface *sfc_font_title,
		SDL_Rect rct_font_title,
		SDL_Surface *sfc_font_menu,
		SDL_Rect rct_font_menu,
		SDL_Surface *sfc_font_parent_menu,
		SDL_Rect rct_font_parent_menu,
		SDL_Surface *sfc_player1,
		SDL_Rect rct_player1,
		SDL_Surface *sfc_player2,
		SDL_Rect rct_player2,
		SDL_Surface *sfc_player3,
		SDL_Rect rct_player3,
		SDL_Surface *sfc_score1,
		SDL_Rect rct_score1,
		SDL_Surface *sfc_score2,
		SDL_Rect rct_score2,
		SDL_Surface *sfc_score3,
		SDL_Rect rct_score3,
		SDL_Surface *sfc_time,
		SDL_Rect rct_time,
		Object object1,
		Game game);

/**
 * \fn int updatePlayer(SDL_Surface *sfc_player3,
 SDL_Rect *rct_player3,
 Object object, Game *game,
 SDL_Rect rct_player1,
 SDL_Rect rct_player2,
 SDL_Surface *sfc_window,
 const int const_time_next_move)
 * \brief update the player position (AI)
 * \param sfc_player3 a pointer to a SDL SDL_Surface structure (the player)
 * \param rct_player3 a pointer to a SDL SDL_Rect structure (the player)
 * \param object an Object structure
 * \param game a pointer to a Game structure (the game)
 * \param rct_player1 a SDL SDL_Rect structure (an other player)
 * \param rct_player2 a SDL SDL_Rect structure (an other player)
 * \param sfc_window a pointer to a Window structure (the window)
 * \param const_time_next_move an integer (the next move time)
 * \return an integer (0 if the player is successfully updated or -1 on error)
 */
int updatePlayer(SDL_Surface *sfc_player3, SDL_Rect *rct_player3, Object object, Game *game, SDL_Rect rct_player1, SDL_Rect rct_player2, SDL_Surface *sfc_window, const int const_time_next_move);

/**
 * \fn int updateObject(Bitmap *bmp_coin,
 Bitmap *bmp_bill,
 Bitmap *bmp_nothing,
 Object *object,
 Window window1,
 Game *game1,
 const int const_time_next_object)
 * \brief update the object
 * \param bmp_coin a pointer to a Bitmap structure (the coin)
 * \param bmp_bill a pointer to a Bitmap structure (the bill)
 * \param bmp_nothing a pointer to a Bitmap structure (nothing)
 * \param object a pointer to a Object structure (the object)
 * \param window1 a window structure (the window)
 * \param game1 a pointer to a Game structure (the game)
 * \param const_time_next_object an integer (the next object time)
 * \return an integer (0 if the object is successfully updated or -1 on error)
 */
int updateObject(Bitmap *bmp_coin, Bitmap *bmp_bill, Bitmap *bmp_nothing, Object *object, Window window1, Game *game1, const int const_time_next_object);

/**
 * \fn int updateScore(Window window1,
 Player *player,
 Mix_Chunk *sample_cool,
 Mix_Chunk *sample_super,
 Object *object,
 int max_score)
 * \brief update the score of one player
 * \param window1 a Window structure (the window)
 * \param player a pointer to a Player structure (the player)
 * \param sample_cool a pointer to a SDL Mix_Chunk structure (the 'cool' sample)
 * \param sample_super a pointer to a SDL Mix_Chunk structure (the 'super' sample)
 * \param object a pointer to an Object structure (the object)
 * \param max_score an interger (the maximum score)
 * \return an integer (0 if the score is successfully updated or -1 on error)
 */
int updateScore(Window window1, Player *player, Mix_Chunk *sample_cool, Mix_Chunk *sample_super, Object *object, int max_score);

/**
 * \fn int updateGameStatus(const int player1_score_value,
 const int player2_score_value,
 const int player3_score_value, 
 Game *game1)
 * \brief update the game status
 * \param player1_score_value an integer (the player 1 score)
 * \param player2_score_value an integer (the player 2 score)
 * \param player3_score_value an integer (the player 3 score)
 * \param game1 a pointer to a Game structure (the game)
 * \return an integer (0 if the game status is successfully updated or -1 on error)
 */
int updateGameStatus(const int player1_score_value, const int player2_score_value, const int player3_score_value, Game *game1);

/**
 * \fn int updateTime(Window window1,
 int font_x, 
 int font_y, 
 Uint8 red, 
 Uint8 green, 
 Uint8 blue, 
 int size, 
 Game *game1,
 const int const_time_next_time_update)
 * \brief update the time
 * \param window1 a Window structure (the window)
 * \param font_x an integer (the font horizontal position)
 * \param font_y an integer (the font vertical position)
 * \param red an integer (the font red color)
 * \param green an integer (the font green color)
 * \param blue an integer (the font blue color)
 * \param size an integer (the font size)
 * \param game1 a pointer to a Game structure (the game)
 * \param const_time_next_time_update (the next time update time)
 * \return an integer (0 if the time is successfully updated or -1 on error)
 */
int updateTime(Window window1, int font_x, int font_y, Uint8 red, Uint8 green, Uint8 blue, int size, Game *game1, const int const_time_next_time_update);

/**
 * \fn void freeAll(SDL_Surface *bmp_nothing,
 SDL_Surface *bmp_bill,
 SDL_Surface *bmp_coin,
 SDL_Surface *bmp_map,
 SDL_Surface *player3,
 SDL_Surface *player3_score,
 SDL_Surface *player2,
 SDL_Surface *player2_score,
 SDL_Surface *player1,
 SDL_Surface *player1_score,
 SDL_Surface *window1,
 TTF_Font *text_title,
 TTF_Font *text_menu,
 TTF_Font *text_parent_menu,
 Mix_Chunk *sample_super,
 Mix_Chunk *sample_cool,
 Mix_Chunk *sample_shopeuros,
 Mix_Chunk *sample_menu_switch,
 Mix_Chunk *sample_menu_return,
 Mix_Music *music1,
 Menu *menu1)
 * \brief free all the memory allocated
 * \param bmp_nothing a pointer to a SDL SDL_Surface structure (the bitmap nothing)
 * \param bmp_bill a pointer to a SDL SDL_Surface structure (the bitmap bill)
 * \param bmp_coin a pointer to a SDL SDL_Surface structure (the bitmap coin)
 * \param bmp_map a pointer to a SDL SDL_Surface structure (the bitmap map)
 * \param player3 a pointer to a SDL SDL_Surface structure (the player 3)
 * \param player3_score a pointer to a SDL SDL_Surface structure (the player 3 score)
 * \param player2  a pointer to a SDL SDL_Surface structure (the player 2)
 * \param player2_score a pointer to a SDL SDL_Surface structure (the player 2 score)
 * \param player1  a pointer to a SDL SDL_Surface structure (the player 1)
 * \param player1_score a pointer to a SDL SDL_Surface structure (the player 1 score)
 * \param window1 a pointer to a SDL SDL_Surface structure (the window)
 * \param text_title a pointer to a SDL TTF_Font structure (the title)
 * \param text_menu a pointer to a SDL TTF_Font structure (the menu)
 * \param text_parent_menu a pointer to a SDL TTF_Font structure (the parent menu)
 * \param sample_super a pointer to a SDL Mix_Chunk structure (the super sample)
 * \param sample_cool a pointer to a SDL Mix_Chunk structure (the cool sample)
 * \param sample_shopeuros a pointer to a SDL Mix_Chunk structure (the shopeuros sample)
 * \param sample_menu_switch a pointer to a SDL Mix_Chunk structure (the menu switch sample)
 * \param sample_menu_return a pointer to a SDL Mix_Chunk structure (the menu return sample)
 * \param music1 a pointer to a SDL Mix_Music structure (the music)
 * \param menu1 a pointer to a Menu structure (the menu)
 * \return nothing
 */
void freeAll(SDL_Surface *bmp_nothing,
		SDL_Surface *bmp_bill,
		SDL_Surface *bmp_coin,
		SDL_Surface *bmp_map,
		SDL_Surface *player3,
		SDL_Surface *player3_score,
		SDL_Surface *player2,
		SDL_Surface *player2_score,
		SDL_Surface *player1,
		SDL_Surface *player1_score,
		SDL_Surface *window1,
		TTF_Font *text_title,
		TTF_Font *text_menu,
		TTF_Font *text_parent_menu,
		Mix_Chunk *sample_super,
		Mix_Chunk *sample_cool,
		Mix_Chunk *sample_shopeuros,
		Mix_Chunk *sample_menu_switch,
		Mix_Chunk *sample_menu_return,
		Mix_Music *music1,
		Menu *menu1);

/**
 * \fn void moveUp(SDL_Rect *rct_player,
 Game game, 
 SDL_Rect rct_other_player1, 
 SDL_Rect rct_other_player2)
 * \brief move a player at the up
 * \param rct_player a pointer to a SDL SDL_Rect structure (the player)
 * \param game a Game structure (the game)
 * \param rct_other_player1 a SDL SDL_Rect structure (an other player)
 * \param rct_other_player2 a SDL SDL_Rect structure (an other player)
 * \return nothing
 */
void moveUp(SDL_Rect *rct_player, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2);

/**
 * \fn void moveDown(SDL_Surface *sfc_player,
 SDL_Rect *rct_player,
 SDL_Surface *sfc_window,
 Game game,
 SDL_Rect rct_other_player1,
 SDL_Rect rct_other_player2)
 * \brief move a player at the down
 * \param sfc_player a pointer to a SDL SDL_Surface structure (the player)
 * \param rct_player a pointer to a SDL SDL_Rect structure (the player)
 * \param sfc_window a pointer to a SDL SDL_Surface structure (the window surface)
 * \param game a Game structure (the game)
 * \param rct_other_player1 a SDL SDL_Rect structure (an other player)
 * \param rct_other_player2 a SDL SDL_Rect structure (an other player)
 * \return nothing
 */
void moveDown(SDL_Surface *sfc_player, SDL_Rect *rct_player, SDL_Surface *sfc_window, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2);

/**
 * \fn void moveLeft(SDL_Rect *rct_player,
 Game game, 
 SDL_Rect rct_other_player1, 
 SDL_Rect rct_other_player2)
 * \brief move a player at the left
 * \param rct_player a pointer to a SDL SDL_Rect structure (the player)
 * \param game a Game structure (the game)
 * \param rct_other_player1 a SDL SDL_Rect structure (an other player)
 * \param rct_other_player2 a SDL SDL_Rect structure (an other player)
 * \return nothing
 */
void moveLeft(SDL_Rect *rct_player, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2);

/**
 * \fn void moveRight(SDL_Surface *sfc_player,
 SDL_Rect *rct_player,
 SDL_Surface *sfc_window, 
 Game game, 
 SDL_Rect rct_other_player1, 
 SDL_Rect rct_other_player2)
 * \brief move a player at the right
 * \param sfc_player a pointer to a SDL SDL_Surface structure (the player)
 * \param rct_player a pointer to a SDL SDL_Rect structure (the player)
 * \param sfc_window a pointer to a SDL SDL_Surface structure (the window surface)
 * \param game a Game structure (the game)
 * \param rct_other_player1 a SDL SDL_Rect structure (an other player)
 * \param rct_other_player2 a SDL SDL_Rect structure (an other player)
 * \return nothing
 */
void moveRight(SDL_Surface *sfc_player, SDL_Rect *rct_player, SDL_Surface *sfc_window, Game game, SDL_Rect rct_other_player1, SDL_Rect rct_other_player2);

#endif
