/**
 * \file structures.h
 * \brief The structures for shopeuros_client
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * \version 20090726
 * \author YuGiOhJCJ <yugiohjcj@1s.fr> http://yugiohjcj.1s.fr/
 */

#ifndef structures_h
#define structures_h

#include <SDL/SDL.h> /* For SDL_Surface and SDL_Rect */
#include <SDL/SDL_ttf.h> /* For TTF_Font */
#include <SDL/SDL_mixer.h> /* For Mix_Chunk and Mix_Music */
#include "enumerations.h" /* For PlayStatus and MenuStatus */

/**
 * \struct font
 * \brief a font
 */
typedef struct font{
	TTF_Font *ttf; /**< a pointer to a SDL TTF_Font structure (the true type font) */
	int size; /**< an integer (the font size) */
	int x; /**< an integer (the font horizontal position) */
	int y; /**< an integer (the font vertical position) */
	SDL_Color color; /**< a SDL SDL_Color structure (the font color) */
	SDL_Surface *sfc; /**< a pointer to a SDL SDL_Surface structure (the font surface) */
	SDL_Rect rct; /**< a SDL SDL_Rect structure (the font rectangular area) */
	char path[65536]; /**< a string (the path to the font) */
} Font; /**< the Font structure */

/**
 * \struct text
 * \brief a text
 */
typedef struct text{
	char string[65536]; /**< a string (the text value) */
	Font font; /**< a Font structure (the text font) */
} Text; /**< the Text structure */

/**
 * \struct score
 * \brief a score
 */
typedef struct score{
	char string[2+3+2+2+1]; /**< a string (the score value) */
	int value; /**< an integer (the score value) */
	Font font; /**< a Font structure (the score font) */
} Score; /**< the Score structure */

/**
 * \struct music
 * \brief a music
 */
typedef struct music{
	Mix_Music *music; /**< a pointer to a SDL Mix_Music structure (the music) */
	char path[65536]; /**< a string (the path to the music file) */
} Music; /**< the Music structure */

/**
 * \struct sample
 * \brief a sample
 */
typedef struct sample{
	Mix_Chunk *sample; /**< a pointer to a SDL Mix_Chunk structure (the sample) */
	char path[65536]; /**< a string (the path to the sample file) */
} Sample; /**< the Sample structure */

/**
 * \struct controls
 * \brief controls
 */
typedef struct controls{
	SDLKey up; /**< a SDLKey structure (the key pressed to move up) */
	SDLKey down; /**< a SDLKey structure (the key pressed to move down) */
	SDLKey left; /**< a SDLKey structure (the key pressed to move left) */
	SDLKey right; /**< a SDLKey structure (the key pressed to move right) */
} Controls; /**< the Controls structure */

/**
 * \struct window
 * \brief a window
 */
typedef struct window{
	int width; /**< an integer (the window width) */
	int height; /**< an integer (the window height) */
	int bpp; /**< an integer (the window bits per pixel) */
	Uint32 flags; /**< an integer (the window flags) */
	char title[16+1]; /**< a string (the window title) */
	char icon[65536]; /**< a string (the path to the window icon) */
	SDL_Surface *sfc; /**< a pointer to a SDL SDL_Surface structure (the window surface) */
	SDL_Surface *sfc_icon; /**< a pointer to a SDL SDL_Surface structure (the window icon surface) */
} Window; /**< the Window structure */

/**
 * \struct bitmap
 * \brief a bitmap
 */
typedef struct bitmap{
	SDL_Surface *sfc; /**< a pointer to a SDL SDL_Surface structure (the bitmap surface) */
	SDL_Rect rct; /**< a SDL SDL_Rect structure (the bitmap rectangular area) */
	char path[65536]; /**< a string (the path to the bitmap) */
} Bitmap; /**< the Bitmap structure */

/**
 * \struct player
 * \brief a player
 */
typedef struct player{
	int money; /**< an integer (the player money) */
	Bitmap bitmap; /**< a Bitmap structure (the player bitmap) */
	Score score; /**< a Score structure (the player score) */
	Controls controls; /**< a Controls structure (the player controls) */
} Player; /**< the Player structure */

/**
 * \struct volume
 * \brief a volume
 */
typedef struct volume{
	int music; /**< an integer (the music volume) */
	int sample; /**< an integer (the sample volume) */
} Volume; /**< the Volume structure */

/**
 * \struct object
 * \brief an object
 */
typedef struct object{
	Bitmap *bitmap; /**< a pointer to a Bitmap structure (the bitmap) */
	ObjectStatus status; /**< an object status (the object status) */
	int value; /**< an integer (the object value) */
} Object; /**< the Object structure */

/**
 * \struct time
 * \brief a time
 */
typedef struct time{
	char string[3+3+3+2+1]; /**< a string (the time value) */
	int value; /**< an interger (the time value in seconds) */
	int max_value; /**< an interger (the maximum time value in seconds) */
	Font font; /**< a Font structure (the time font) */
} Time; /**< the Time structure */

/**
 * \struct game
 * \brief a game
 */
typedef struct game{
	Uint32 time_current; /**< an integer (the game current time, in ms) */
	Uint32 time_next_move; /**< an integer (the game next move time, in ms) */
	Uint32 time_next_object; /**< an integer (the game next object time, in ms) */
	Uint32 time_next_time_update; /**< an integer (the game next time update time, in ms) */
	Uint32 time_next_time_is_up; /**< an integer (the game next time is up time, in ms) */
	MenuStatus status_menu; /**< a menu status (the game menu status) */
	PlayStatus status_play; /**< a play status (the game play status) */
	Time time; /**< a Time structure (the game time) */
	int max_time; /**< an integer (the game maximum time, in ms) */
	int max_score; /**< an integer (the game maximum score) */
	Difficulty difficulty; /**< a difficulty (the game difficulty) */
	Mode mode; /**< a mode (the game mode) */
	int move_increment; /**< an integer (the game move increment) */
} Game; /**< the Game structure */

#endif
